
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>


enum {
//	_xinfreq = 20_000_000,
	_xtlfreq = 20_000_000,
	_clkfreq = 250_000_000,
	DOWNLOAD_BAUD = 230_400,
	DEBUG_BAUD = DOWNLOAD_BAUD,

};



static uint32_t  randfill( uint32_t *addr, size_t size )
{
	uint32_t  ticks;

	size >>= 2;
	ticks = _cnt();
	do {
		*(addr++) = _rnd();
	} while( --size );

	return( _cnt() - ticks );
}


static int  compare( uint32_t *addr1, uint32_t *addr2, size_t size )
{
	size_t  pass = size;

	pass >>= 2;
	do {
		if( *(addr1++) != *(addr2++) )  break;
	} while( --pass );

	return( size - (pass << 2) );
}


static void  tester( const char *filename, const size_t size )
{
	FILE  *fh;
	size_t  count;
	uint32_t  ticks, *data1, *data2;

	if( data1 = __builtin_alloca( size ) )  printf( "addr1 = 0x%x  ", data1 );
	else {
		printf( "\n malloc() #1 failed!\n" );
		return;
	}
	if( data2 = __builtin_alloca( size ) )  printf( "addr2 = 0x%x  ", data2 );
	else {
		printf( "\n malloc() #2 failed!\n" );
		return;
	}
	printf( " Randfill ticks = %d\n", randfill( data1, size ) );

	if( fh = fopen( filename, "w" ) )
	{
		ticks = _cnt();
		count = fwrite( data1, 1, size, fh );
		fclose( fh );
		ticks = _cnt() - ticks;
		printf( " Written %d of %d bytes at %d kB/s\n", count, size,
			(_muldiv64( count, _clockfreq(), ticks ) + 512) >> 10 );
	} else  printf( "\n SD card write error!\n" );

	if( fh = fopen( filename, "r" ) )
	{
		ticks = _cnt();
		count = fread( data2, 1, size, fh );
		fclose( fh );
		ticks = _cnt() - ticks;
		printf( " Read %d of %d bytes at %d kB/s  ", count, size,
			(_muldiv64( count, _clockfreq(), ticks ) + 512) >> 10 );

		count = compare( data1, data2, size );
		if( count == size )  printf( "Matches!  :)\n" );
		else    printf( "Mis-match! :(   Offset %d\n", count );

	} else  printf( "\n SD card read error!\n" );

//	free( data2 );
//	free( data1 );
}


//===================================================================================

void  main( void )
{
	uint8_t  buff[] = { 0xaa,0x08,0xcc,0x53 };
	int PIN_SS = 60;
	int  i;
	FILE  *fh;

	printf( "   clkfreq = %d   clkmode = 0x%x\n", _clockfreq(), _clockmode() );
	mount( "/sd", _vfs_open_sdcard() );

	tester( "/sd/speed1.bin", 100000 );
	tester( "/sd/speed2.bin", 100000 );
	tester( "/sd/speed3.bin", 100000 );
	tester( "/sd/speed4.bin", 100000 );

	for( i = 0; i < 5; i ++ )  tester( "/sd/speed5.bin", 212000 );
	for( i = 0; i < 5; i ++ )  tester( "/sd/speed6.bin", 20000 );
	for( i = 0; i < 5; i ++ )  tester( "/sd/speed7.bin", 2000 );
	for( i = 0; i < 5; i ++ )  tester( "/sd/speed8.bin", 200 );

	printf( "\n" );  // blank line for re-run separation
	_waitms( 500 );
	_clkset( 0, 25_000_000 );  // cool running
}
